
# q-expansion of newform 2240.2.g.j, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2240
weight = 2
poly_data = [9, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 0, 1, 0], 3], [[0, 0, 0, 1], 3]]

hecke_ring_character_values = [[1471, [1, 0, 0, 0]], [1541, [1, 0, 0, 0]], [897, [-1, 0, 0, 0]], [1921, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 1, 0, 1], [-1, 0, -1, -1], [0, 0, -1, 0], [0, -2, 0, 2], [0, -1, 2, -1], [0, 0, 2, 0], [4, 1, 0, -1], [0, 2, 2, 2], [2, -2, 0, 2], [4, -2, 0, 2], [0, 0, -2, 0], [-6, -2, 0, 2], [0, 2, 4, 2], [0, -2, 4, -2], [0, 2, -6, 2], [-4, -1, 0, 1], [-6, -1, 0, 1], [0, 0, 8, 0], [-6, -2, 0, 2], [0, 2, 2, 2], [-2, 2, 0, -2], [0, 1, 0, 1], [10, 0, 0, 0], [0, -4, 6, -4], [-6, -1, 0, 1], [0, 2, -8, 2], [0, 0, 8, 0], [2, -2, 0, 2], [0, 4, -10, 4], [0, 2, -10, 2], [4, -1, 0, 1], [0, 4, 8, 4], [4, 1, 0, -1], [6, 4, 0, -4], [0, -8, 0, 8], [0, -1, -6, -1], [0, -2, -12, -2], [0, 2, 0, 2], [0, 5, 6, 5], [4, -4, 0, 4], [-2, -5, 0, 5], [-2, -6, 0, 6], [0, -8, 2, -8], [0, 2, -14, 2], [-12, 2, 0, -2], [-12, 0, 0, 0], [0, 0, 4, 0], [0, 3, 0, 3], [-2, 7, 0, -7], [0, 4, -20, 4], [16, 4, 0, -4], [6, 6, 0, -6], [4, -1, 0, 1], [0, 6, 6, 6], [0, 4, 0, 4], [2, -7, 0, 7], [12, 0, 0, 0], [0, -2, 10, -2], [-18, 0, 0, 0], [0, 5, 16, 5], [0, -5, 6, -5], [0, -5, 8, -5], [12, 0, 0, 0], [0, -8, 2, -8], [0, -10, -2, -10], [4, -6, 0, 6], [0, -8, -10, -8], [0, 2, -24, 2], [-6, 1, 0, -1], [0, 2, -18, 2], [8, -8, 0, 8], [0, 0, 32, 0], [0, 6, 10, 6], [16, -6, 0, 6], [0, 2, 12, 2], [18, 2, 0, -2], [0, 3, 10, 3], [0, 12, 0, -12], [-10, 10, 0, -10], [-4, -1, 0, 1], [14, 4, 0, -4], [8, 4, 0, -4], [0, 4, 10, 4], [-8, -12, 0, 12], [0, -4, 0, -4], [10, 0, 0, 0], [0, -8, 10, -8], [-10, 3, 0, -3], [0, 8, 16, 8], [0, 1, -8, 1], [-24, -6, 0, 6], [0, -6, -22, -6], [0, 8, 0, -8], [16, 4, 0, -4], [0, 0, 4, 0], [-34, -1, 0, 1], [18, -6, 0, 6], [0, 7, -16, 7], [10, 8, 0, -8], [0, -6, 4, -6], [0, 6, 2, 6], [0, 9, -8, 9], [-24, 4, 0, -4], [16, 2, 0, -2], [0, 10, 22, 10], [0, 7, 16, 7], [0, -2, 6, -2], [18, 2, 0, -2], [-10, 12, 0, -12], [0, -12, 4, -12], [0, 4, -18, 4], [0, -8, 10, -8], [-44, -1, 0, 1], [-18, 10, 0, -10], [16, -4, 0, 4], [0, -7, 8, -7], [0, 14, -12, 14], [0, -4, -30, -4], [12, -2, 0, 2], [14, -11, 0, 11], [0, -4, 8, -4], [0, -1, 34, -1], [0, -8, -16, -8], [-4, 7, 0, -7], [-26, -6, 0, 6], [6, 14, 0, -14], [-20, -10, 0, 10], [0, -6, 8, -6], [0, -17, -2, -17], [-20, -10, 0, 10], [0, -6, 30, -6], [-32, 4, 0, -4], [0, -4, 42, -4], [-26, -2, 0, 2], [26, -6, 0, 6], [0, -3, -6, -3], [0, 1, -48, 1], [0, -11, -26, -11], [-28, 8, 0, -8], [12, 11, 0, -11], [-18, -4, 0, 4], [0, -16, 0, -16], [0, -8, -4, -8], [-2, 17, 0, -17], [-32, 2, 0, -2], [0, 13, -2, 13], [0, 16, -14, 16], [12, -17, 0, 17], [0, -12, 16, -12], [0, -12, 10, -12], [18, 4, 0, -4], [0, 4, 32, 4], [0, -6, -12, -6], [0, -4, 32, -4], [-16, 8, 0, -8], [22, -2, 0, 2], [-6, -14, 0, 14], [0, -2, -46, -2], [22, 1, 0, -1], [0, 18, 0, 18], [0, -4, -12, -4], [0, 14, -2, 14], [12, 1, 0, -1], [0, 12, 0, 12], [0, 6, 28, 6], [46, 6, 0, -6], [0, 3, -50, 3], [-4, -16, 0, 16], [0, -5, 26, -5], [36, 4, 0, -4], [18, 5, 0, -5], [-14, 6, 0, -6], [0, 20, 4, 20], [-4, -6, 0, 6], [-6, 16, 0, -16], [-32, 0, 0, 0], [-22, -10, 0, 10], [0, 12, 12, 12], [2, -17, 0, 17], [0, 2, 50, 2], [-20, 13, 0, -13], [0, 0, 26, 0], [0, 2, 30, 2], [0, -6, -10, -6], [18, -3, 0, 3], [0, 14, -6, 14], [0, 13, 8, 13], [-22, 12, 0, -12], [-24, -14, 0, 14], [0, -10, 14, -10], [0, -12, -12, -12], [20, -12, 0, 12], [-18, 11, 0, -11], [0, -12, 20, -12], [0, -10, -26, -10], [-4, 16, 0, -16], [0, -12, 18, -12], [0, 2, 10, 2], [0, 4, -40, 4], [-2, 12, 0, -12], [40, 2, 0, -2], [0, -5, -42, -5], [6, 24, 0, -24], [12, 13, 0, -13], [0, 17, 26, 17], [12, -2, 0, 2], [0, -6, -28, -6], [0, 0, 0, 0], [-20, 3, 0, -3], [0, -16, -22, -16], [14, 9, 0, -9], [0, -6, 30, -6], [0, -5, 48, -5], [32, -12, 0, 12], [-46, -2, 0, 2], [0, 8, 24, 8], [-54, 6, 0, -6], [0, 0, -48, 0], [0, 0, 26, 0], [-34, 2, 0, -2], [-8, 8, 0, -8], [30, 0, 0, 0], [0, -2, 6, -2], [0, 3, -40, 3], [-18, -2, 0, 2], [0, -6, -30, -6], [-8, -12, 0, 12], [0, 0, -28, 0], [-12, -10, 0, 10], [0, -20, -14, -20], [-12, 17, 0, -17], [2, 10, 0, -10], [18, -16, 0, 16], [0, 1, -40, 1], [0, -16, -12, -16], [-22, 2, 0, -2], [0, 10, 6, 10], [-48, 8, 0, -8], [12, 0, 0, 0], [0, 18, -12, 18], [-52, 5, 0, -5], [0, 0, -16, 0], [30, 0, 0, 0], [0, -2, -54, -2], [-12, 22, 0, -22], [0, 0, -28, 0], [-12, -15, 0, 15], [-48, -2, 0, 2], [0, 8, -24, 8], [0, 2, 26, 2], [30, -8, 0, 8], [0, -12, -16, -12], [22, -12, 0, 12], [0, 15, 6, 15], [16, 4, 0, -4], [10, -2, 0, 2], [0, 15, 8, 15], [0, -1, 54, -1], [0, 12, 10, 12], [0, 6, -2, 6], [0, -10, -12, -10], [58, 7, 0, -7], [0, 17, 34, 17], [0, -4, 26, -4], [28, 17, 0, -17], [18, 12, 0, -12], [10, 12, 0, -12], [0, -8, 24, -8], [0, -20, 6, -20], [-34, 7, 0, -7], [0, -2, 0, -2], [0, -6, 10, -6], [-8, 8, 0, -8], [0, -14, 6, -14], [0, 0, 24, 0], [0, -6, 24, -6], [70, 0, 0, 0], [32, 0, 0, 0], [-36, -11, 0, 11], [0, 16, -12, 16], [-14, -14, 0, 14], [0, -18, 20, -18], [-38, 11, 0, -11], [0, 9, -16, 9], [0, -8, 0, 8], [0, -4, 68, -4], [0, 2, 66, 2], [52, -2, 0, 2], [-30, 0, 0, 0], [-6, 4, 0, -4], [0, -7, 0, -7], [0, 16, 18, 16], [76, -3, 0, 3], [0, -6, 22, -6], [-14, 9, 0, -9], [-56, -12, 0, 12], [0, -1, 2, -1], [12, -17, 0, 17], [0, 15, 8, 15], [0, -12, 26, -12], [0, -20, -22, -20], [-30, 10, 0, -10]]
